// Copyright (c) 2009, Jens Peter Secher <jpsecher@gmail.com>
//
// Permission to use, copy, modify, and/or distribute this software for any
// purpose with or without fee is hereby granted, provided that the above
// copyright notice and this permission notice appear in all copies.
//
// THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
// WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
// ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
// ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
// OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

class RegExpUtilUnitTest extends haxe.unit.TestCase
{
	public function testEscpaing()
	{
		assertEquals( RegExpUtil.escape( "", RegExpPlace.Whole ), "" );
		assertEquals
		(
			RegExpUtil.escape( "^a.b*c+d?e[f]g(h)i\\j$", RegExpPlace.Whole ),
			"\\^a\\.b\\*c\\+d\\?e\\[f\\]g\\(h\\)i\\\\j\\$"
		);
		assertEquals
		(
			RegExpUtil.escape( "^$" ),
			"^$"
		);
		assertEquals
		(
			RegExpUtil.escape( "^$", RegExpPlace.Initial ),
			"\\^$"
		);
		assertEquals
		(
			RegExpUtil.escape( "^$", RegExpPlace.Final ),
			"^\\$"
		);
		assertEquals
		(
			RegExpUtil.escape( "^.*+?[]()\\$", RegExpPlace.Whole ),
			"\\^\\.\\*\\+\\?\\[\\]\\(\\)\\\\\\$"
		);
	}
}


